@@ -9,7 +9,6 @@ use crate::{
     connection_status::{ConnectionState, ConnectionStep},
     consumer::Consumer,
     consumers::Consumers,
-    executor::Executor,
     frames::{ExpectedReply, Frames},
     internal_rpc::InternalRPCHandle,
     message::{BasicGetMessage, BasicReturnMessage, Delivery},
@@ -22,18 +21,27 @@ use crate::{
     topology::RestoredChannel,
     topology_internal::ChannelDefinitionInternal,
     types::*,
-    BasicProperties, Configuration, ConfirmationPromise, Connection, ConnectionStatus, DeliveryTag,
-    Error, ExchangeKind, Promise, PromiseChain, PromiseResolver, Result,
+    BasicProperties, Configuration, Connection, ConnectionStatus, Error, ExchangeKind, Promise,
+    PromiseResolver, Result,
 };
 use amq_protocol::frame::{AMQPContentHeader, AMQPFrame};
-use log::{debug, error, info, log_enabled, trace, Level::Trace};
-use parking_lot::Mutex;
+use executor_trait::FullExecutor;
 use serde::{Deserialize, Serialize};
 use std::{convert::TryFrom, fmt, sync::Arc};
+use tracing::{error, info, level_enabled, trace, Level};
 
+/// Main entry point for most AMQP operations.
+///
+/// It serves as a lightweight connection and can be obtained from a
+///  [`Connection`] by calling [`Connection::create_channel`].
+///
+/// See also the RabbitMQ documentation on [channels](https://www.rabbitmq.com/channels.html).
+///
+/// [`Connection`]: ./struct.Connection.html
+/// [`Connection::create_channel`]: ./struct.Connection.html#method.create_channel
 #[derive(Clone)]
 pub struct Channel {
-    id: u16,
+    id: ChannelId,
     configuration: Configuration,
     status: ChannelStatus,
     connection_status: ConnectionStatus,
@@ -46,7 +54,7 @@ pub struct Channel {
     waker: SocketStateHandle,
     internal_rpc: InternalRPCHandle,
     frames: Frames,
-    executor: Arc<dyn Executor>,
+    executor: Arc<dyn FullExecutor + Send + Sync>,
     channel_closer: Option<Arc<ChannelCloser>>,
     connection_closer: Option<Arc<ConnectionCloser>>,
 }
@@ -69,22 +77,20 @@ impl fmt::Debug for Channel {
             .field("basic_get_delivery", &self.basic_get_delivery)
             .field("returned_messages", &self.returned_messages)
             .field("frames", &self.frames)
-            .field("executor", &self.executor)
             .finish()
     }
 }
 
 impl Channel {
-    #[allow(clippy::too_many_arguments)]
     pub(crate) fn new(
-        channel_id: u16,
+        channel_id: ChannelId,
         configuration: Configuration,
         connection_status: ConnectionStatus,
         global_registry: Registry,
         waker: SocketStateHandle,
         internal_rpc: InternalRPCHandle,
         frames: Frames,
-        executor: Arc<dyn Executor>,
+        executor: Arc<dyn FullExecutor + Send + Sync>,
         connection_closer: Option<Arc<ConnectionCloser>>,
     ) -> Channel {
         let returned_messages = ReturnedMessages::default();
@@ -194,45 +200,43 @@ impl Channel {
         self.set_state(ChannelState::Closing);
         if let Some(error) = error {
             self.error_publisher_confirms(error.clone());
-            let _ = self.error_consumers(error.clone()); // ignore the error here, only happens with default executor if we cannot spawn a thread
+            let _ = self.error_consumers(error); // ignore the error here, only happens with default executor if we cannot spawn a thread
         } else {
             self.consumers.start_cancel();
         }
     }
 
-    fn set_closed(&self, error: Error) -> Result<()> {
+    fn set_closed(&self, error: Error) {
         self.set_state(ChannelState::Closed);
         self.error_publisher_confirms(error.clone());
-        let res = self.cancel_consumers();
+        self.cancel_consumers();
         self.internal_rpc.remove_channel(self.id, error);
-        res
     }
 
-    fn set_error(&self, error: Error) -> Result<()> {
+    fn set_error(&self, error: Error) {
         self.set_state(ChannelState::Error);
         self.error_publisher_confirms(error.clone());
-        let res = self.error_consumers(error.clone());
+        self.error_consumers(error.clone());
         self.internal_rpc.remove_channel(self.id, error);
-        res
     }
 
     pub(crate) fn error_publisher_confirms(&self, error: Error) {
         self.acknowledgements.on_channel_error(error);
     }
 
-    pub(crate) fn cancel_consumers(&self) -> Result<()> {
-        self.consumers.cancel()
+    pub(crate) fn cancel_consumers(&self) {
+        self.consumers.cancel();
     }
 
-    pub(crate) fn error_consumers(&self, error: Error) -> Result<()> {
-        self.consumers.error(error)
+    pub(crate) fn error_consumers(&self, error: Error) {
+        self.consumers.error(error);
     }
 
     pub(crate) fn set_state(&self, state: ChannelState) {
         self.status.set_state(state);
     }
 
-    pub fn id(&self) -> u16 {
+    pub fn id(&self) -> ChannelId {
         self.id
     }
 
@@ -258,17 +262,17 @@ impl Channel {
     }
 
     fn wake(&self) {
-        trace!("channel {} wake", self.id);
+        trace!(channel=%self.id, "wake");
         self.waker.wake()
     }
 
-    fn assert_channel0(&self, class_id: u16, method_id: u16) -> Result<()> {
+    fn assert_channel0(&self, class_id: Identifier, method_id: Identifier) -> Result<()> {
         if self.id == 0 {
             Ok(())
         } else {
             error!(
-                "Got a connection frame on channel {}, closing connection",
-                self.id
+                channel=%self.id,
+                "Got a connection frame on, closing connection"
             );
             let error = AMQPError::new(
                 AMQPHardError::COMMANDINVALID.into(),
@@ -284,47 +288,48 @@ impl Channel {
         }
     }
 
-    pub fn close(&self, reply_code: ShortUInt, reply_text: &str) -> Promise<()> {
-        self.do_channel_close(reply_code, reply_text, 0, 0)
+    pub async fn close(&self, reply_code: ReplyCode, reply_text: &str) -> Result<()> {
+        self.do_channel_close(reply_code, reply_text, 0, 0).await
     }
 
-    pub fn basic_consume(
+    pub async fn basic_consume(
         &self,
         queue: &str,
         consumer_tag: &str,
         options: BasicConsumeOptions,
         arguments: FieldTable,
-    ) -> PromiseChain<Consumer> {
+    ) -> Result<Consumer> {
         self.do_basic_consume(queue, consumer_tag, options, arguments, None)
+            .await
     }
 
-    pub fn basic_get(
+    pub async fn basic_get(
         &self,
         queue: &str,
         options: BasicGetOptions,
-    ) -> PromiseChain<Option<BasicGetMessage>> {
-        self.do_basic_get(queue, options, None)
+    ) -> Result<Option<BasicGetMessage>> {
+        self.do_basic_get(queue, options, None).await
     }
 
-    pub fn exchange_declare(
+    pub async fn exchange_declare(
         &self,
         exchange: &str,
         kind: ExchangeKind,
         options: ExchangeDeclareOptions,
         arguments: FieldTable,
-    ) -> Promise<()> {
+    ) -> Result<()> {
         self.do_exchange_declare(exchange, kind.kind(), options, arguments, kind.clone())
+            .await
     }
 
-    pub fn wait_for_confirms(&self) -> ConfirmationPromise<Vec<BasicReturnMessage>> {
-        if let Some(promise) = self.acknowledgements.get_last_pending() {
+    pub async fn wait_for_confirms(&self) -> Result<Vec<BasicReturnMessage>> {
+        if let Some(last_pending) = self.acknowledgements.get_last_pending() {
             trace!("Waiting for pending confirms");
-            let returned_messages = self.returned_messages.clone();
-            promise.traverse(move |_| Ok(returned_messages.drain()))
+            last_pending.await?;
         } else {
             trace!("No confirms to wait for");
-            ConfirmationPromise::new_with_data(Ok(Vec::default()))
         }
+        Ok(self.returned_messages.drain())
     }
 
     #[cfg(test)]
@@ -357,23 +362,22 @@ impl Channel {
         resolver: PromiseResolver<()>,
         expected_reply: Option<ExpectedReply>,
     ) {
-        trace!("channel {} send_frame", self.id);
+        trace!(channel=%self.id, "send_frame");
         self.frames.push(self.id, frame, resolver, expected_reply);
         self.wake();
     }
 
-    fn send_method_frame_with_body(
+    async fn send_method_frame_with_body(
         &self,
         method: AMQPClass,
-        payload: Vec<u8>,
+        payload: &[u8],
         properties: BasicProperties,
         publisher_confirms_result: Option<PublisherConfirm>,
-    ) -> Result<PromiseChain<PublisherConfirm>> {
+    ) -> Result<PublisherConfirm> {
         let class_id = method.get_amqp_class_id();
         let header = AMQPContentHeader {
             class_id,
-            weight: 0,
-            body_size: payload.len() as u64,
+            body_size: payload.len() as PayloadSize,
             properties,
         };
         let frame_max = self.configuration.frame_max();
@@ -385,32 +389,25 @@ impl Channel {
         // a content body frame 8 bytes of overhead
         frames.extend(
             payload
-                .as_slice()
                 .chunks(frame_max as usize - 8)
                 .map(|chunk| AMQPFrame::Body(self.id, chunk.into())),
         );
 
-        // tweak to make rustc happy
-        let data = Arc::new(Mutex::new((
-            publisher_confirms_result,
-            self.returned_messages.clone(),
-        )));
-
-        trace!("channel {} send_frames", self.id);
+        trace!(channel=%self.id, "send_frames");
         let promise = self.frames.push_frames(frames);
         self.wake();
-        Ok(promise.traverse(move |res| {
-            res.map(|()| {
-                let mut data = data.lock();
-                data.0
-                    .take()
-                    .unwrap_or_else(|| PublisherConfirm::not_requested(data.1.clone()))
-            })
-        }))
+        promise.await?;
+        Ok(publisher_confirms_result
+            .unwrap_or_else(|| PublisherConfirm::not_requested(self.returned_messages.clone())))
     }
 
-    fn handle_invalid_contents(&self, error: String, class_id: u16, method_id: u16) -> Result<()> {
-        error!("{}", error);
+    fn handle_invalid_contents(
+        &self,
+        error: String,
+        class_id: Identifier,
+        method_id: Identifier,
+    ) -> Result<()> {
+        error!(%error);
         let error = AMQPError::new(AMQPHardError::UNEXPECTEDFRAME.into(), error.into());
         self.internal_rpc.close_connection(
             error.get_id(),
@@ -423,40 +420,33 @@ impl Channel {
 
     pub(crate) fn handle_content_header_frame(
         &self,
-        class_id: u16,
-        size: LongLongUInt,
+        class_id: Identifier,
+        size: PayloadSize,
         properties: BasicProperties,
     ) -> Result<()> {
         self.status.set_content_length(
             self.id,
             class_id,
             size,
-            |delivery_cause, confirm_mode| {
-                match delivery_cause {
-                    DeliveryCause::Consume(consumer_tag) => {
-                        self.consumers.handle_content_header_frame(
-                            &self,
-                            consumer_tag,
-                            size,
-                            properties,
-                        )?;
-                    }
-                    DeliveryCause::Get => {
-                        self.basic_get_delivery
-                            .handle_content_header_frame(size, properties);
-                    }
-                    DeliveryCause::Return => {
-                        self.returned_messages.handle_content_header_frame(
-                            size,
-                            properties,
-                            confirm_mode,
-                        );
-                    }
+            |delivery_cause, confirm_mode| match delivery_cause {
+                DeliveryCause::Consume(consumer_tag) => {
+                    self.consumers
+                        .handle_content_header_frame(consumer_tag, size, properties);
+                }
+                DeliveryCause::Get => {
+                    self.basic_get_delivery
+                        .handle_content_header_frame(size, properties);
+                }
+                DeliveryCause::Return => {
+                    self.returned_messages.handle_content_header_frame(
+                        size,
+                        properties,
+                        confirm_mode,
+                    );
                 }
-                Ok(())
             },
             |msg| {
-                error!("{}", msg);
+                error!(%msg);
                 let error = AMQPError::new(AMQPHardError::FRAMEERROR.into(), msg.into());
                 self.internal_rpc.close_connection(
                     error.get_id(),
@@ -464,7 +454,9 @@ impl Channel {
                     class_id,
                     0,
                 );
-                self.set_error(Error::ProtocolError(error))
+                let error = Error::ProtocolError(error);
+                self.set_error(error.clone());
+                Err(error)
             },
             |msg| self.handle_invalid_contents(msg, class_id, 0),
         )
@@ -473,30 +465,20 @@ impl Channel {
     pub(crate) fn handle_body_frame(&self, payload: Vec<u8>) -> Result<()> {
         self.status.receive(
             self.id,
-            payload.len() as LongLongUInt,
-            |delivery_cause, remaining_size, confirm_mode| {
-                match delivery_cause {
-                    DeliveryCause::Consume(consumer_tag) => {
-                        self.consumers.handle_body_frame(
-                            &self,
-                            consumer_tag,
-                            remaining_size,
-                            payload,
-                        )?;
-                    }
-                    DeliveryCause::Get => {
-                        self.basic_get_delivery
-                            .handle_body_frame(remaining_size, payload);
-                    }
-                    DeliveryCause::Return => {
-                        self.returned_messages.handle_body_frame(
-                            remaining_size,
-                            payload,
-                            confirm_mode,
-                        );
-                    }
+            payload.len() as PayloadSize,
+            |delivery_cause, remaining_size, confirm_mode| match delivery_cause {
+                DeliveryCause::Consume(consumer_tag) => {
+                    self.consumers
+                        .handle_body_frame(consumer_tag, remaining_size, payload);
+                }
+                DeliveryCause::Get => {
+                    self.basic_get_delivery
+                        .handle_body_frame(remaining_size, payload);
+                }
+                DeliveryCause::Return => {
+                    self.returned_messages
+                        .handle_body_frame(remaining_size, payload, confirm_mode);
                 }
-                Ok(())
             },
             |msg| self.handle_invalid_contents(msg, 0, 0),
         )
@@ -522,16 +504,26 @@ impl Channel {
         self.consumers.start_cancel_one(consumer_tag);
     }
 
-    fn acknowledgement_error(&self, error: AMQPError, class_id: u16, method_id: u16) -> Result<()> {
+    fn acknowledgement_error(
+        &self,
+        error: AMQPError,
+        class_id: Identifier,
+        method_id: Identifier,
+    ) -> Result<()> {
         error!("Got a bad acknowledgement from server, closing channel");
-        self.internal_rpc
-            .register_internal_future(self.do_channel_close(
-                error.get_id(),
-                error.get_message().as_str(),
-                class_id,
-                method_id,
-            ))?;
-        Err(Error::ProtocolError(error))
+        let channel = self.clone();
+        let err = error.clone();
+        self.internal_rpc.register_internal_future(async move {
+            channel
+                .do_channel_close(
+                    error.get_id(),
+                    error.get_message().as_str(),
+                    class_id,
+                    method_id,
+                )
+                .await
+        });
+        Err(Error::ProtocolError(err))
     }
 
     fn on_connection_start_ok_sent(
@@ -539,25 +531,22 @@ impl Channel {
         resolver: PromiseResolver<Connection>,
         connection: Connection,
         credentials: Credentials,
-    ) -> Result<()> {
+    ) {
         self.connection_status
             .set_connection_step(ConnectionStep::StartOk(resolver, connection, credentials));
-        Ok(())
     }
 
-    fn on_connection_open_sent(&self, resolver: PromiseResolver<Connection>) -> Result<()> {
+    fn on_connection_open_sent(&self, resolver: PromiseResolver<Connection>) {
         self.connection_status
             .set_connection_step(ConnectionStep::Open(resolver));
-        Ok(())
     }
 
-    fn on_connection_close_ok_sent(&self, error: Error) -> Result<()> {
+    fn on_connection_close_ok_sent(&self, error: Error) {
         if let Error::ProtocolError(_) = error {
             self.internal_rpc.set_connection_error(error);
         } else {
             self.internal_rpc.set_connection_closed(error);
         }
-        Ok(())
     }
 
     fn next_expected_close_ok_reply(&self) -> Option<Reply> {
@@ -569,31 +558,32 @@ impl Channel {
         self.set_closing(None);
     }
 
-    fn on_channel_close_ok_sent(&self, error: Error) -> Result<()> {
-        self.set_closed(error)
+    fn on_channel_close_ok_sent(&self, error: Error) {
+        self.set_closed(error);
     }
 
-    fn on_basic_recover_async_sent(&self) -> Result<()> {
-        self.consumers.drop_prefetched_messages()
+    fn on_basic_recover_async_sent(&self) {
+        self.consumers.drop_prefetched_messages();
     }
 
-    fn on_basic_ack_sent(&self, multiple: bool, delivery_tag: DeliveryTag) -> Result<()> {
+    fn on_basic_ack_sent(&self, multiple: bool, delivery_tag: DeliveryTag) {
         if multiple && delivery_tag == 0 {
-            self.consumers.drop_prefetched_messages()
-        } else {
-            Ok(())
+            self.consumers.drop_prefetched_messages();
         }
     }
 
-    fn on_basic_nack_sent(&self, multiple: bool, delivery_tag: DeliveryTag) -> Result<()> {
+    fn on_basic_nack_sent(&self, multiple: bool, delivery_tag: DeliveryTag) {
         if multiple && delivery_tag == 0 {
-            self.consumers.drop_prefetched_messages()
-        } else {
-            Ok(())
+            self.consumers.drop_prefetched_messages();
         }
     }
 
-    fn tune_connection_configuration(&self, channel_max: u16, frame_max: u32, heartbeat: u16) {
+    fn tune_connection_configuration(
+        &self,
+        channel_max: ChannelId,
+        frame_max: FrameSize,
+        heartbeat: Heartbeat,
+    ) {
         // If we disable the heartbeat (0) but the server don't, follow it and enable it too
         // If both us and the server want heartbeat enabled, pick the lowest value.
         if self.configuration.heartbeat() == 0
@@ -612,7 +602,7 @@ impl Channel {
             }
         }
         if self.configuration.channel_max() == 0 {
-            self.configuration.set_channel_max(u16::max_value());
+            self.configuration.set_channel_max(ChannelId::max_value());
         }
 
         if frame_max != 0 {
@@ -623,12 +613,12 @@ impl Channel {
             }
         }
         if self.configuration.frame_max() == 0 {
-            self.configuration.set_frame_max(u32::max_value());
+            self.configuration.set_frame_max(FrameSize::max_value());
         }
     }
 
     fn on_connection_start_received(&self, method: protocol::connection::Start) -> Result<()> {
-        trace!("Server sent connection::Start: {:?}", method);
+        trace!(?method, "Server sent connection::Start");
         let state = self.connection_status.state();
         if let (
             ConnectionState::Connecting,
@@ -646,13 +636,19 @@ impl Channel {
 
             if !method
                 .mechanisms
+                .to_string()
                 .split_whitespace()
                 .any(|m| m == mechanism_str)
             {
-                error!("unsupported mechanism: {}", mechanism);
+                error!(%mechanism, "unsupported mechanism");
             }
-            if !method.locales.split_whitespace().any(|l| l == locale) {
-                error!("unsupported locale: {}", mechanism);
+            if !method
+                .locales
+                .to_string()
+                .split_whitespace()
+                .any(|l| l == locale)
+            {
+                error!(%locale, "unsupported locale");
             }
 
             if !options.client_properties.contains_key("product")
@@ -687,18 +683,23 @@ impl Channel {
                 .client_properties
                 .insert("capabilities".into(), AMQPValue::FieldTable(capabilities));
 
-            self.internal_rpc
-                .register_internal_future(self.connection_start_ok(
-                    options.client_properties,
-                    &mechanism_str,
-                    &credentials.sasl_auth_string(mechanism),
-                    &locale,
-                    resolver,
-                    connection,
-                    credentials,
-                ))
+            let channel = self.clone();
+            self.internal_rpc.register_internal_future(async move {
+                channel
+                    .connection_start_ok(
+                        options.client_properties,
+                        &mechanism_str,
+                        &credentials.sasl_auth_string(mechanism),
+                        &locale,
+                        resolver,
+                        connection,
+                        credentials,
+                    )
+                    .await
+            });
+            Ok(())
         } else {
-            error!("Invalid state: {:?}", state);
+            error!(?state, "Invalid state");
             let error = Error::InvalidConnectionState(state);
             self.internal_rpc.set_connection_error(error.clone());
             Err(error)
@@ -706,17 +707,21 @@ impl Channel {
     }
 
     fn on_connection_secure_received(&self, method: protocol::connection::Secure) -> Result<()> {
-        trace!("Server sent connection::Secure: {:?}", method);
+        trace!(?method, "Server sent connection::Secure");
 
         let state = self.connection_status.state();
         if let (ConnectionState::Connecting, Some(ConnectionStep::StartOk(.., credentials))) =
             (state.clone(), self.connection_status.connection_step())
         {
-            self.internal_rpc.register_internal_future(
-                self.connection_secure_ok(&credentials.rabbit_cr_demo_answer()),
-            )
+            let channel = self.clone();
+            self.internal_rpc.register_internal_future(async move {
+                channel
+                    .connection_secure_ok(&credentials.rabbit_cr_demo_answer())
+                    .await
+            });
+            Ok(())
         } else {
-            error!("Invalid state: {:?}", state);
+            error!(?state, "Invalid state");
             let error = Error::InvalidConnectionState(state);
             self.internal_rpc.set_connection_error(error.clone());
             Err(error)
@@ -724,7 +729,7 @@ impl Channel {
     }
 
     fn on_connection_tune_received(&self, method: protocol::connection::Tune) -> Result<()> {
-        debug!("Server sent Connection::Tune: {:?}", method);
+        trace!(?method, "Server sent Connection::Tune");
 
         let state = self.connection_status.state();
         if let (
@@ -738,20 +743,22 @@ impl Channel {
                 method.heartbeat,
             );
 
-            self.internal_rpc
-                .register_internal_future(self.connection_tune_ok(
-                    self.configuration.channel_max(),
-                    self.configuration.frame_max(),
-                    self.configuration.heartbeat(),
-                ))?;
-            self.internal_rpc
-                .register_internal_future(self.connection_open(
-                    &self.connection_status.vhost(),
-                    connection,
-                    resolver,
-                ))
+            let channel = self.clone();
+            let configuration = self.configuration.clone();
+            let vhost = self.connection_status.vhost();
+            self.internal_rpc.register_internal_future(async move {
+                channel
+                    .connection_tune_ok(
+                        configuration.channel_max(),
+                        configuration.frame_max(),
+                        configuration.heartbeat(),
+                    )
+                    .await?;
+                channel.connection_open(&vhost, connection, resolver).await
+            });
+            Ok(())
         } else {
-            error!("Invalid state: {:?}", state);
+            error!(?state, "Invalid state");
             let error = Error::InvalidConnectionState(state);
             self.internal_rpc.set_connection_error(error.clone());
             Err(error)
@@ -771,7 +778,7 @@ impl Channel {
             resolver.swear(Ok(connection));
             Ok(())
         } else {
-            error!("Invalid state: {:?}", state);
+            error!(?state, "Invalid state");
             let error = Error::InvalidConnectionState(state);
             self.internal_rpc.set_connection_error(error.clone());
             Err(error)
@@ -782,14 +789,16 @@ impl Channel {
         let error = AMQPError::try_from(method.clone())
             .map(|error| {
                 error!(
-                    "Connection closed on channel {} by {}:{} => {:?} => {}",
-                    self.id, method.class_id, method.method_id, error, method.reply_text
+                    channel=%self.id,
+                    ?method,
+                    ?error,
+                    "Connection closed",
                 );
                 Error::ProtocolError(error)
             })
             .unwrap_or_else(|error| {
-                error!("{}", error);
-                info!("Connection closed on channel {}: {:?}", self.id, method);
+                error!(%error);
+                info!(channel=%self.id, ?method, "Connection closed");
                 Error::InvalidConnectionState(ConnectionState::Closed)
             });
         self.internal_rpc.set_connection_closing();
@@ -834,10 +843,15 @@ impl Channel {
 
     fn on_channel_flow_received(&self, method: protocol::channel::Flow) -> Result<()> {
         self.status.set_send_flow(method.active);
-        self.internal_rpc
-            .register_internal_future(self.channel_flow_ok(ChannelFlowOkOptions {
-                active: method.active,
-            }))
+        let channel = self.clone();
+        self.internal_rpc.register_internal_future(async move {
+            channel
+                .channel_flow_ok(ChannelFlowOkOptions {
+                    active: method.active,
+                })
+                .await
+        });
+        Ok(())
     }
 
     fn on_channel_flow_ok_received(
@@ -853,23 +867,26 @@ impl Channel {
     fn on_channel_close_received(&self, method: protocol::channel::Close) -> Result<()> {
         let error = AMQPError::try_from(method.clone()).map(|error| {
             error!(
-                "Channel closed on channel {} by {}:{} => {:?} => {}",
-                self.id, method.class_id, method.method_id, error, method.reply_text
+                channel=%self.id, ?method, ?error,
+                "Channel closed"
             );
             Error::ProtocolError(error)
         });
         self.set_closing(error.clone().ok());
         let error = error.unwrap_or_else(|error| {
-            error!("{}", error);
-            info!("Channel closed on channel {}: {:?}", self.id, method);
+            error!(%error);
+            info!(channel=%self.id, ?method, "Channel closed");
             Error::InvalidChannelState(ChannelState::Closing)
         });
+        let channel = self.clone();
         self.internal_rpc
-            .register_internal_future(self.channel_close_ok(error))
+            .register_internal_future(async move { channel.channel_close_ok(error).await });
+        Ok(())
     }
 
     fn on_channel_close_ok_received(&self) -> Result<()> {
-        self.set_closed(Error::InvalidChannelState(ChannelState::Closed))
+        self.set_closed(Error::InvalidChannelState(ChannelState::Closed));
+        Ok(())
     }
 
     fn on_exchange_bind_ok_received(
@@ -922,7 +939,7 @@ impl Channel {
     fn on_queue_delete_ok_received(
         &self,
         method: protocol::queue::DeleteOk,
-        resolver: PromiseResolver<LongUInt>,
+        resolver: PromiseResolver<MessageCount>,
         queue: ShortString,
     ) -> Result<()> {
         self.local_registry.deregister_queue(queue.as_str());
@@ -934,7 +951,7 @@ impl Channel {
     fn on_queue_purge_ok_received(
         &self,
         method: protocol::queue::PurgeOk,
-        resolver: PromiseResolver<LongUInt>,
+        resolver: PromiseResolver<MessageCount>,
     ) -> Result<()> {
         resolver.swear(Ok(method.message_count));
         Ok(())
@@ -1090,18 +1107,19 @@ impl Channel {
     }
 
     fn on_basic_cancel_received(&self, method: protocol::basic::Cancel) -> Result<()> {
-        self.consumers
-            .deregister(&method.consumer_tag)
-            .and(if !method.nowait {
-                self.internal_rpc
-                    .register_internal_future(self.basic_cancel_ok(method.consumer_tag.as_str()))
-            } else {
-                Ok(())
-            })
+        self.consumers.deregister(method.consumer_tag.as_str());
+        if !method.nowait {
+            let channel = self.clone();
+            self.internal_rpc.register_internal_future(async move {
+                channel.basic_cancel_ok(method.consumer_tag.as_str()).await
+            });
+        }
+        Ok(())
     }
 
     fn on_basic_cancel_ok_received(&self, method: protocol::basic::CancelOk) -> Result<()> {
-        self.consumers.deregister(&method.consumer_tag)
+        self.consumers.deregister(method.consumer_tag.as_str());
+        Ok(())
     }
 
     fn on_basic_ack_received(&self, method: protocol::basic::Ack) -> Result<()> {
@@ -1181,7 +1199,8 @@ impl Channel {
     }
 
     fn on_basic_recover_ok_received(&self) -> Result<()> {
-        self.consumers.drop_prefetched_messages()
+        self.consumers.drop_prefetched_messages();
+        Ok(())
     }
 
     fn on_confirm_select_ok_received(&self) -> Result<()> {
